@@ -36,6 +36,7 @@
 #include "glade-cursor.h"
 #include "glade-catalog.h"
 #include "glade-design-view.h"
+#include "glade-design-layout.h"
 #include "glade-marshallers.h"
 #include "glade-accumulators.h"
 
@@ -55,6 +56,7 @@
 enum
 {
   DOC_SEARCH,
+  SIGNAL_EDITOR_CREATED,
   LAST_SIGNAL
 };
 
@@ -298,6 +300,14 @@ glade_app_init (GladeApp * app)
   priv->config = glade_app_config_load (app);
 }
 
+static void
+glade_app_event_handler (GdkEvent *event, gpointer data)
+{
+  if (glade_app_do_event (event)) return;
+
+  gtk_main_do_event (event);
+}
+
 static void
 glade_app_class_init (GladeAppClass * klass)
 {
@@ -330,13 +340,63 @@ glade_app_class_init (GladeAppClass * klass)
                     G_TYPE_NONE, 3,
                     G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
+  /**
+   * GladeApp::signal-editor-created:
+   * @gladeapp: the #GladeApp which received the signal.
+   * @signal_editor: the new #GladeSignalEditor.
+   *
+   * Emitted when a new signal editor created.
+   * A tree view is created in the default handler.
+   * Connect your handler before the default handler for setting a custom column or renderer
+   * and after it for connecting to the tree view signals
+   */
+  glade_app_signals[SIGNAL_EDITOR_CREATED] =
+    g_signal_new ("signal-editor-created",
+                  G_TYPE_FROM_CLASS (object_class),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  glade_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1, G_TYPE_OBJECT);  
+
   g_type_class_add_private (klass, sizeof (GladeAppPrivate));
+
+  gdk_event_handler_set (glade_app_event_handler, NULL, NULL);
 }
 
 /*****************************************************************
  *                       Public API                              *
  *****************************************************************/
 
+/**
+ * glade_app_do_event:
+ * @event: the event to process.
+ *
+ * This function has to be called in an event handler for widget selection to work.
+ * See gdk_event_handler_set()
+ *
+ * Returns: true if the event was handled.
+ */ 
+gboolean
+glade_app_do_event (GdkEvent *event)
+{
+  GdkWindow *window = event->any.window;
+  GtkWidget *layout;
+  gpointer widget;
+
+  if (window == NULL) return FALSE;
+    
+  gdk_window_get_user_data (window, &widget);
+
+  /* As a slight optimization we could replace gtk_widget_get_ancestor()
+   * with a custom function that only iterates trought parents with windows.
+   */
+  if (widget && IS_GLADE_WIDGET_EVENT (event->type) &&
+      (layout = gtk_widget_get_ancestor (widget, GLADE_TYPE_DESIGN_LAYOUT)))
+    return _glade_design_layout_do_event (GLADE_DESIGN_LAYOUT (layout), event);
+
+  return FALSE;
+}
+
 /**
  * glade_app_config_save
  *
@@ -456,7 +516,10 @@ GladeApp *
 glade_app_get (void)
 {
   if (!singleton_app)
-    g_critical ("No available GladeApp");
+    {
+      singleton_app = glade_app_new ();
+    }
+
   return singleton_app;
 }
 
