@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* TODO : s/glade_xml_get_/glade_xml_node_get/g */
 #ifndef __GLADE_XML_UTILS_H__
 #define __GLADE_XML_UTILS_H__
@@ -19,35 +18,26 @@ typedef struct _GladeXmlContext GladeXmlContext;
 typedef struct _GladeXmlNode    GladeXmlNode;
 typedef struct _GladeXmlDoc     GladeXmlDoc;
 
+/* FIXME: forward declarations to get around circular header dependencies.
+ */
+typedef struct _GladeWidget         GladeWidget;
+typedef struct _GladeWidgetAdaptor  GladeWidgetAdaptor;
+typedef struct _GladeProperty       GladeProperty;
+typedef struct _GladeProject        GladeProject;
 
-#define GLADE_XML_TAG_PROJECT(type) \
-	((type == GLADE_PROJECT_FORMAT_LIBGLADE) ? \
-	 GLADE_XML_TAG_LIBGLADE_PROJECT : GLADE_XML_TAG_BUILDER_PROJECT)
 
-#define GLADE_XML_TAG_WIDGET(type) \
-	((type == GLADE_PROJECT_FORMAT_LIBGLADE) ? \
-	 GLADE_XML_TAG_LIBGLADE_WIDGET : GLADE_XML_TAG_BUILDER_WIDGET)
+#define GLADE_GTKBUILDER_VERSIONING_BASE_MAJOR    2
+#define GLADE_GTKBUILDER_VERSIONING_BASE_MINOR    14
+#define GLADE_GTKBUILDER_HAS_VERSIONING(maj, min) \
+	((maj) >= GLADE_GTKBUILDER_VERSIONING_BASE_MAJOR && \
+	 (min) >= GLADE_GTKBUILDER_VERSIONING_BASE_MINOR)
+
 
 /* Used for catalog tags and attributes */
-
-
-	gdouble                    version_since; /* Version in which this widget was
-						   * introduced
-						   */
-	gboolean                   deprecated; /* If this widget is currently
-							* deprecated
-							*/
-	gboolean                   builder_unsupported; /* If this widget is not supported
-							 * by gtkbuilder
-							 */
-
-#define GLADE_XML_TAG_LIBGLADE_PROJECT            "glade-interface"
-#define GLADE_XML_TAG_BUILDER_PROJECT             "interface"
-#define GLADE_XML_TAG_LIBGLADE_WIDGET             "widget"
-#define GLADE_XML_TAG_BUILDER_WIDGET              "object"
+#define GLADE_XML_TAG_PROJECT                     "interface"
+#define GLADE_XML_TAG_WIDGET                      "object"
 
 #define GLADE_XML_TAG_VERSION                     "version"
-#define GLADE_XML_TAG_REQUIRES_LIBGLADE_EXTRA     "requires-version"
 #define GLADE_XML_TAG_REQUIRES                    "requires"
 #define GLADE_XML_TAG_LIB                         "lib"
 #define GLADE_XML_TAG_PROPERTY                    "property"
@@ -56,6 +46,7 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_XML_TAG_SIGNAL                      "signal"
 #define GLADE_XML_TAG_HANDLER                     "handler"
 #define GLADE_XML_TAG_AFTER                       "after"
+#define GLADE_XML_TAG_SWAPPED                     "swapped"
 #define GLADE_XML_TAG_OBJECT                      "object"
 #define GLADE_XML_TAG_NAME                        "name"
 #define GLADE_XML_TAG_CHILD                       "child"
@@ -64,14 +55,18 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_XML_TAG_INTERNAL_CHILD              "internal-child"
 #define GLADE_XML_TAG_I18N_TRUE                   "yes"
 #define GLADE_XML_TAG_SIGNAL_TRUE                 "yes"
+#define GLADE_XML_TAG_SIGNAL_FALSE                "no"
 #define GLADE_XML_TAG_TYPE                        "type"
-
+#define GLADE_XML_TAG_SOURCES                     "sources"
+#define GLADE_XML_TAG_SOURCE                      "source"
+#define GLADE_XML_TAG_FILENAME                    "filename"
+#define GLADE_XML_TAG_STOCK_ID                    "stock-id"
 
 #define GLADE_TAG_VERSION                         "version"
 #define GLADE_TAG_TARGETABLE                      "targetable"
 #define GLADE_TAG_VERSION_SINCE                   "since"
+#define GLADE_TAG_BUILDER_SINCE                   "gtkbuilder-since"
 #define GLADE_TAG_DEPRECATED                      "deprecated"
-#define GLADE_TAG_BUILDER_UNSUPPORTED             "builder-unsupported"
 
 #define GLADE_TAG_GLADE_CATALOG                   "glade-catalog"
 #define GLADE_TAG_GLADE_WIDGET_CLASSES            "glade-widget-classes"
@@ -86,13 +81,19 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_SIGNALS                         "signals"
 #define GLADE_TAG_SIGNAL                          "signal"
 #define GLADE_TAG_DEFAULT                         "default"
+#define GLADE_TAG_PARENTLESS_WIDGET               "parentless-widget"
 #define GLADE_TAG_DISABLED                        "disabled"
+#define GLADE_TAG_CONSTRUCT_ONLY                  "construct-only"
+#define GLADE_TAG_NEEDS_SYNC                      "needs-sync"
 #define GLADE_TAG_DEFAULT_PALETTE_STATE           "default-palette-state"
 #define GLADE_TAG_REPLACE_CHILD_FUNCTION          "replace-child-function"
+#define GLADE_TAG_CREATE_WIDGET_FUNCTION          "create-widget-function"
+#define GLADE_TAG_CONSTRUCT_OBJECT_FUNCTION       "construct-object-function"
 #define GLADE_TAG_DEEP_POST_CREATE_FUNCTION       "deep-post-create-function"
 #define GLADE_TAG_POST_CREATE_FUNCTION            "post-create-function"
 #define GLADE_TAG_GET_INTERNAL_CHILD_FUNCTION     "get-internal-child-function"
 #define GLADE_TAG_ADD_CHILD_FUNCTION              "add-child-function"
+#define GLADE_TAG_ADD_CHILD_VERIFY_FUNCTION       "add-child-verify-function"
 #define GLADE_TAG_REMOVE_CHILD_FUNCTION           "remove-child-function"
 #define GLADE_TAG_GET_CHILDREN_FUNCTION           "get-children-function"
 #define GLADE_TAG_CHILD_SET_PROP_FUNCTION         "child-set-property-function"
@@ -101,12 +102,15 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_CONSTRUCTOR_FUNCTION            "constructor-function"
 #define GLADE_TAG_ACTION_ACTIVATE_FUNCTION        "action-activate-function"
 #define GLADE_TAG_CHILD_ACTION_ACTIVATE_FUNCTION  "child-action-activate-function"
+#define GLADE_TAG_ACTION_SUBMENU_FUNCTION         "action-submenu-function"
+#define GLADE_TAG_DEPENDS_FUNCTION                "depends-function"
 #define GLADE_TAG_READ_WIDGET_FUNCTION            "read-widget-function"
 #define GLADE_TAG_WRITE_WIDGET_FUNCTION           "write-widget-function"
 #define GLADE_TAG_READ_CHILD_FUNCTION             "read-child-function"
 #define GLADE_TAG_WRITE_CHILD_FUNCTION            "write-child-function"
 #define GLADE_TAG_CREATE_EPROP_FUNCTION           "create-editor-property-function"
 #define GLADE_TAG_STRING_FROM_VALUE_FUNCTION      "string-from-value-function"
+#define GLADE_TAG_CREATE_EDITABLE_FUNCTION        "create-editable-function"
 #define GLADE_TAG_PROPERTIES                      "properties"
 #define GLADE_TAG_PACKING_PROPERTIES              "packing-properties"
 #define GLADE_TAG_PROPERTY                        "property"
@@ -115,9 +119,11 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_ACTION                          "action"
 #define GLADE_TAG_TYPE                            "type"
 #define GLADE_TAG_SPEC                            "spec"
+#define GLADE_TAG_SPECIFICATIONS                  "parameter-spec"
+#define GLADE_TAG_MAX_VALUE                       "max"
+#define GLADE_TAG_MIN_VALUE                       "min"
+#define GLADE_TAG_VALUE_TYPE                      "value-type"
 #define GLADE_TAG_TOOLTIP                         "tooltip"
-#define GLADE_TAG_PARAMETERS                      "parameters"
-#define GLADE_TAG_PARAMETER                       "parameter"
 #define GLADE_TAG_SET_FUNCTION                    "set-property-function"
 #define GLADE_TAG_GET_FUNCTION                    "get-property-function"
 #define GLADE_TAG_VERIFY_FUNCTION                 "verify-function"
@@ -126,6 +132,7 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_OPTIONAL                        "optional"
 #define GLADE_TAG_OPTIONAL_DEFAULT                "optional-default"
 #define GLADE_TAG_VISIBLE                         "visible"
+#define GLADE_TAG_CUSTOM_LAYOUT                   "custom-layout"
 #define GLADE_TAG_EXPANDED                        "expanded"
 #define GLADE_TAG_GENERIC_NAME                    "generic-name"
 #define GLADE_TAG_NAME                            "name"
@@ -136,6 +143,7 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_VALUE                           "value"
 #define GLADE_TAG_TRANSLATABLE                    "translatable"
 #define GLADE_TAG_HAS_CONTEXT                     "context"
+#define GLADE_TAG_CONTEXT                         "context"
 #define GLADE_TAG_COMMENT                         "comments"
 #define GLADE_TAG_PACKING_DEFAULTS                "packing-defaults"
 #define GLADE_TAG_PARENT_CLASS                    "parent-class"
@@ -147,15 +155,15 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_SAVE_ALWAYS                     "save-always"
 #define GLADE_TAG_EDITABLE                        "editable"
 #define GLADE_TAG_IGNORE                          "ignore"
-#define GLADE_TAG_VISIBLE_LINES                   "visible-lines"
+#define GLADE_TAG_MULTILINE                       "multiline"
 #define GLADE_TAG_RESOURCE                        "resource"
+#define GLADE_TAG_STOCK                           "stock"
+#define GLADE_TAG_STOCK_ICON                      "stock-icon"
 #define GLADE_TAG_THEMED_ICON                     "themed-icon"
 #define GLADE_TAG_INIT_FUNCTION                   "init-function"
 #define GLADE_TAG_ATK_PROPERTY                    "atk-property"
-#define GLADE_TAG_FIXED                           "fixed"
 #define GLADE_TAG_TRANSFER_ON_PASTE               "transfer-on-paste"
 #define GLADE_TAG_WEIGHT                          "weight"
-#define GLADE_TAG_STOCK                           "stock"
 #define GLADE_TAG_GET_TYPE_FUNCTION               "get-type-function"
 #define GLADE_TAG_TOPLEVEL                        "toplevel"
 #define GLADE_TAG_USE_PLACEHOLDERS                "use-placeholders"
@@ -169,7 +177,7 @@ typedef struct _GladeXmlDoc     GladeXmlDoc;
 #define GLADE_TAG_ICON_PREFIX                     "icon-prefix"
 #define GLADE_TAG_ICON_NAME                       "icon-name"
 #define GLADE_TAG_IMPORTANT                       "important"
-#define GLADE_ENUM_DATA_TAG                       "GladeEnumDataTag"
+#define GLADE_TAG_CREATE_TYPE                     "create-type"
 
 
 /* search child */
@@ -199,7 +207,7 @@ gchar *  glade_xml_get_property_string (GladeXmlNode *node_in, const gchar *name
 gboolean glade_xml_get_property_boolean (GladeXmlNode *node_in, const gchar *name, gboolean _default);
 gdouble  glade_xml_get_property_double (GladeXmlNode *node_in, const gchar *name, gdouble _default);
 gint     glade_xml_get_property_int (GladeXmlNode *node_in, const gchar *name, gint _default);
-gboolean glade_xml_get_property_version (GladeXmlNode *node_in, const gchar *name, gint *major, gint *minor);
+gboolean glade_xml_get_property_version (GladeXmlNode *node_in, const gchar *name, guint16 *major, guint16 *minor);
 GList   *glade_xml_get_property_targetable_versions (GladeXmlNode *node_in, const gchar *name);
 
 
@@ -211,6 +219,7 @@ GladeXmlNode * glade_xml_node_new (GladeXmlContext *context, const gchar *name);
 GladeXmlNode * glade_xml_node_new_comment (GladeXmlContext *context, const gchar *comment);
 void           glade_xml_node_delete (GladeXmlNode *node);
 GladeXmlNode * glade_xml_node_get_children (GladeXmlNode *node);
+GladeXmlNode * glade_xml_node_get_parent (GladeXmlNode *node_in);
 GladeXmlNode * glade_xml_node_next (GladeXmlNode *node_in);
 gboolean       glade_xml_node_verify (GladeXmlNode * node_in, const gchar *name);
 gboolean       glade_xml_node_verify_silent (GladeXmlNode *node_in, const gchar *name);
@@ -238,6 +247,9 @@ GladeXmlContext * glade_xml_context_new_from_path (const gchar *full_path,
 						   const gchar *root_name);
 GladeXmlDoc *     glade_xml_context_get_doc (GladeXmlContext *context);
 
+/* Dumps an xml string from a context */
+gchar * glade_xml_dump_from_context (GladeXmlContext *context);
+
 gboolean        glade_xml_load_sym_from_node (GladeXmlNode     *node_in,
 					      GModule          *module,
 					      gchar            *tagname,
