@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2008 Tristan Van Berkom.
  *
- * This library is free software; you can redistribute it and/or it
+ * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
@@ -59,7 +59,6 @@ struct _GladeImageEditor
 	GList *properties;         /* A list of eprops to update at load() time */
 
 	gboolean loading;          /* Loading flag for loading widgets in the editor */
-	gboolean modifying;        /* Flag for monitoring project changes */
 };
 
 struct _GladeImageEditorClass
